@@ -19,10 +19,9 @@ package collector
 import (
 	"errors"
 	"fmt"
+	"log/slog"
 	"os"
 
-	"github.com/go-kit/log"
-	"github.com/go-kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/procfs/sysfs"
 )
@@ -35,7 +34,7 @@ type thermalZoneCollector struct {
 	coolingDeviceCurState *prometheus.Desc
 	coolingDeviceMaxState *prometheus.Desc
 	zoneTemp              *prometheus.Desc
-	logger                log.Logger
+	logger                *slog.Logger
 }
 
 func init() {
@@ -43,7 +42,7 @@ func init() {
 }
 
 // NewThermalZoneCollector returns a new Collector exposing kernel/system statistics.
-func NewThermalZoneCollector(logger log.Logger) (Collector, error) {
+func NewThermalZoneCollector(logger *slog.Logger) (Collector, error) {
 	fs, err := sysfs.NewFS(*sysPath)
 	if err != nil {
 		return nil, fmt.Errorf("failed to open sysfs: %w", err)
@@ -74,7 +73,7 @@ func (c *thermalZoneCollector) Update(ch chan<- prometheus.Metric) error {
 	thermalZones, err := c.fs.ClassThermalZoneStats()
 	if err != nil {
 		if errors.Is(err, os.ErrNotExist) || errors.Is(err, os.ErrPermission) || errors.Is(err, os.ErrInvalid) {
-			level.Debug(c.logger).Log("msg", "Could not read thermal zone stats", "err", err)
+			c.logger.Debug("Could not read thermal zone stats", "err", err)
 			return ErrNoData
 		}
 		return err
