@@ -67,6 +67,7 @@ enabled_collectors=$(cat << COLLECTORS
   netstat
   nfs
   nfsd
+  pcidevice
   pressure
   processes
   qdisc
@@ -325,6 +326,7 @@ get "127.0.0.1:${port}/metrics" | grep --text -E -v "${skip_re}" > "${generated_
 non_deterministic_metrics=$(cat << METRICS
   node_boot_time_seconds
   node_cpu_frequency_hertz
+  node_cpu_frequency_max_hertz
   node_cpu_seconds_total
   node_disk_io_time_seconds_total
   node_disk_read_bytes_total
@@ -363,6 +365,33 @@ non_deterministic_metrics=$(cat << METRICS
   node_network_receive_bytes_total
   node_network_receive_multicast_total
   node_network_transmit_multicast_total
+  node_zfs_abdstats_linear_count_total
+  node_zfs_abdstats_linear_data_bytes
+  node_zfs_abdstats_scatter_chunk_waste_bytes
+  node_zfs_abdstats_scatter_count_total
+  node_zfs_abdstats_scatter_data_bytes
+  node_zfs_abdstats_struct_bytes
+  node_zfs_arcstats_anon_bytes
+  node_zfs_arcstats_c_bytes
+  node_zfs_arcstats_c_max_bytes
+  node_zfs_arcstats_data_bytes
+  node_zfs_arcstats_demand_data_hits_total
+  node_zfs_arcstats_demand_data_misses_total
+  node_zfs_arcstats_demand_metadata_hits_total
+  node_zfs_arcstats_demand_metadata_misses_total
+  node_zfs_arcstats_hdr_bytes
+  node_zfs_arcstats_hits_total
+  node_zfs_arcstats_meta_bytes
+  node_zfs_arcstats_mfu_bytes
+  node_zfs_arcstats_mfu_ghost_hits_total
+  node_zfs_arcstats_misses_total
+  node_zfs_arcstats_mru_bytes
+  node_zfs_arcstats_mru_ghost_bytes
+  node_zfs_arcstats_other_bytes
+  node_zfs_arcstats_pd_bytes
+  node_zfs_arcstats_size_bytes
+  node_zfs_zfetchstats_hits_total
+  node_zfs_zfetchstats_misses_total
 METRICS
 )
 
